@@ -24,13 +24,37 @@
  */
 class CControllerNotificationsGet extends CController {
 
+	/**
+	 * @var array
+	 */
+	private $notifications = [];
+
+	/**
+	 * @var array
+	 */
+	private $settings = [];
+
+	/**
+	 * @var int
+	 */
+	private $timeout_time = 0;
+
+	/**
+	 * @var int
+	 */
+	private $time_from = 0;
+
+	/**
+	 * @var array
+	 */
+	private $known_eventids = [];
+
 	protected function init() {
 		parent::init();
 
 		$this->notifications = [];
 		$this->settings = getMessageSettings();
-		$config = select_config();
-		$ok_timeout = (int) timeUnitToSeconds($config['ok_period']);
+		$ok_timeout = (int) timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::OK_PERIOD));
 		$timeout = (int) timeUnitToSeconds($this->settings['timeout']);
 		$this->settings['timeout'] = $timeout;
 		$this->settings['ok_timeout'] = min([$timeout, $ok_timeout]);
@@ -189,14 +213,14 @@ class CControllerNotificationsGet extends CController {
 
 				$url_problems = (new CUrl('zabbix.php'))
 					->setArgument('action', 'problem.view')
-					->setArgument('filter_hostids[]', $trigger['hosts'][0]['hostid'])
 					->setArgument('filter_set', '1')
+					->setArgument('hostids[]', $trigger['hosts'][0]['hostid'])
 					->getUrl();
 
 				$url_events = (new CUrl('zabbix.php'))
 					->setArgument('action', 'problem.view')
-					->setArgument('filter_triggerids[]', $triggerid)
 					->setArgument('filter_set', '1')
+					->setArgument('triggerids[]', $triggerid)
 					->getUrl();
 
 				$url_trigger_events_pt = (new CUrl('tr_events.php'))->setArgument('triggerid', $triggerid);
