@@ -70,21 +70,21 @@ function graph_item_drawtype2str($drawtype) {
 
 function graph_item_aggr_fnc2str($calc_fnc) {
 	switch ($calc_fnc) {
-		case GRAPH_AGGREGATE_NONE:
+		case AGGREGATE_NONE:
 			return _('none');
-		case GRAPH_AGGREGATE_MIN:
+		case AGGREGATE_MIN:
 			return _('min');
-		case GRAPH_AGGREGATE_MAX:
+		case AGGREGATE_MAX:
 			return _('max');
-		case GRAPH_AGGREGATE_AVG:
+		case AGGREGATE_AVG:
 			return _('avg');
-		case GRAPH_AGGREGATE_COUNT:
+		case AGGREGATE_COUNT:
 			return _('count');
-		case GRAPH_AGGREGATE_SUM:
+		case AGGREGATE_SUM:
 			return _('sum');
-		case GRAPH_AGGREGATE_FIRST:
+		case AGGREGATE_FIRST:
 			return _('first');
-		case GRAPH_AGGREGATE_LAST:
+		case AGGREGATE_LAST:
 			return _('last');
 	}
 }
@@ -150,7 +150,7 @@ function getGraphByGraphId($graphId) {
 		return $dbGraph;
 	}
 
-	error(_s('No graph item with graphid "%1$s".', $graphId));
+	error(_s('No graph item with graph ID "%1$s".', $graphId));
 
 	return false;
 }
@@ -286,10 +286,11 @@ function getGraphParentTemplates(array $graphs, $flag) {
  * @param string $graphid
  * @param array  $parent_templates  The list of the templates, prepared by getGraphParentTemplates() function.
  * @param int    $flag              Origin of the graph (ZBX_FLAG_DISCOVERY_NORMAL or ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array|null
  */
-function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
+function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag, bool $provide_links) {
 	if (!array_key_exists($graphid, $parent_templates['links'])) {
 		return null;
 	}
@@ -300,8 +301,8 @@ function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
 
 	$template = $parent_templates['templates'][$parent_templates['links'][$graphid]['hostid']];
 
-	if ($template['permission'] == PERM_READ_WRITE) {
-		$url = (new CUrl('graphs.php'));
+	if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
+		$url = (new CUrl('graphs.php'))->setArgument('context', 'template');
 
 		if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 			$url->setArgument('parent_discoveryid', $parent_templates['links'][$graphid]['lld_ruleid']);
@@ -313,10 +314,10 @@ function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
 				->setArgument('filter_hostids', [$template['hostid']]);
 		}
 
-		$name = (new CLink(CHtml::encode($template['name']), $url))->addClass(ZBX_STYLE_LINK_ALT);
+		$name = (new CLink($template['name'], $url))->addClass(ZBX_STYLE_LINK_ALT);
 	}
 	else {
-		$name = new CSpan(CHtml::encode($template['name']));
+		$name = new CSpan($template['name']);
 	}
 
 	return [$name->addClass(ZBX_STYLE_GREY), NAME_DELIMITER];
@@ -328,17 +329,20 @@ function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
  * @param string $graphid
  * @param array  $parent_templates  The list of the templates, prepared by getGraphParentTemplates() function.
  * @param int    $flag              Origin of the item (ZBX_FLAG_DISCOVERY_NORMAL or ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array
  */
-function makeGraphTemplatesHtml($graphid, array $parent_templates, $flag) {
+function makeGraphTemplatesHtml($graphid, array $parent_templates, $flag, bool $provide_links) {
 	$list = [];
 
 	while (array_key_exists($graphid, $parent_templates['links'])) {
 		$template = $parent_templates['templates'][$parent_templates['links'][$graphid]['hostid']];
 
-		if ($template['permission'] == PERM_READ_WRITE) {
-			$url = (new CUrl('graphs.php'))->setArgument('form', 'update');
+		if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
+			$url = (new CUrl('graphs.php'))
+				->setArgument('form', 'update')
+				->setArgument('context', 'template');
 
 			if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 				$url->setArgument('parent_discoveryid', $parent_templates['links'][$graphid]['lld_ruleid']);
@@ -350,13 +354,13 @@ function makeGraphTemplatesHtml($graphid, array $parent_templates, $flag) {
 				$url->setArgument('hostid', $template['hostid']);
 			}
 
-			$name = new CLink(CHtml::encode($template['name']), $url);
+			$name = new CLink($template['name'], $url);
 		}
 		else {
-			$name = (new CSpan(CHtml::encode($template['name'])))->addClass(ZBX_STYLE_GREY);
+			$name = (new CSpan($template['name']))->addClass(ZBX_STYLE_GREY);
 		}
 
-		array_unshift($list, $name, '&nbsp;&rArr;&nbsp;');
+		array_unshift($list, $name, [NBSP(), RARR(), NBSP()]);
 
 		$graphid = $parent_templates['links'][$graphid]['graphid'];
 	}
@@ -537,12 +541,16 @@ function get_next_color($palettetype = 0) {
  * @param resource 	$image
  * @param int		$fontsize
  * @param int 		$angle
- * @param int		$x
- * @param int 		$y
+ * @param int|float $x
+ * @param int|float $y
  * @param int		$color		a numeric color identifier from imagecolorallocate() or imagecolorallocatealpha()
  * @param string	$string
  */
 function imageText($image, $fontsize, $angle, $x, $y, $color, $string) {
+	$x = (int) $x;
+	$y = (int) $y;
+	$string = strtr($string, ['&' => '&#38;']);
+
 	if ((preg_match(ZBX_PREG_DEF_FONT_STRING, $string) && $angle != 0) || ZBX_FONT_NAME == ZBX_GRAPH_FONT_NAME) {
 		$ttf = ZBX_FONTPATH.'/'.ZBX_FONT_NAME.'.ttf';
 		imagettftext($image, $fontsize, $angle, $x, $y, $color, $ttf, $string);
@@ -583,6 +591,8 @@ function imageText($image, $fontsize, $angle, $x, $y, $color, $string) {
  * @return array
  */
 function imageTextSize($fontsize, $angle, $string) {
+	$string = strtr($string, ['&' => '&#38;']);
+
 	if (preg_match(ZBX_PREG_DEF_FONT_STRING, $string) && $angle != 0) {
 		$ttf = ZBX_FONTPATH.'/'.ZBX_FONT_NAME.'.ttf';
 	}
@@ -761,6 +771,7 @@ function yieldGraphScaleInterval(float $min, float $max, bool $is_binary, int $p
 * @param float $data_min   Minimum extreme of the graph.
 * @param float $data_max   Maximum extreme of the graph.
 * @param bool  $is_binary  Is the scale binary (use 1024 base for units)?
+* @param bool  $calc_power Should scale power be calculated?
 * @param bool  $calc_min   Should scale minimum be calculated?
 * @param bool  $calc_max   Should scale maximum be calculated?
 * @param int   $rows_min   Minimum number of scale rows.
@@ -768,15 +779,15 @@ function yieldGraphScaleInterval(float $min, float $max, bool $is_binary, int $p
 *
 * @return array|null
 */
-function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_binary, bool $calc_min, bool $calc_max,
-		int $rows_min, int $rows_max): ?array {
+function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_binary, bool $calc_power,
+		bool $calc_min, bool $calc_max, int $rows_min, int $rows_max): ?array {
 	$scale_min = truncateFloat($data_min);
 	$scale_max = truncateFloat($data_max);
 
 	if ($scale_min >= $scale_max) {
 		if ($scale_max > 0) {
 			if ($calc_min) {
-				$scale_min = $scale_max > 0 ? 0 : ($scale_max == 0 ? -1 : $scale_max * 1.25);
+				$scale_min = 0;
 			}
 			elseif ($calc_max) {
 				$scale_max = $scale_min < 0 ? 0 : ($scale_min == 0 ? 1 : $scale_min * 1.25);
@@ -790,7 +801,7 @@ function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_
 				$scale_max = $scale_min < 0 ? 0 : ($scale_min == 0 ? 1 : $scale_min * 1.25);
 			}
 			elseif ($calc_min) {
-				$scale_min = $scale_max > 0 ? 0 : ($scale_max == 0 ? -1 : $scale_max * 1.25);
+				$scale_min = $scale_max == 0 ? -1 : $scale_max * 1.25;
 			}
 			else {
 				return null;
@@ -798,7 +809,9 @@ function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_
 		}
 	}
 
-	$power = (int) min(8, max(0, floor(log(max(abs($scale_min), abs($scale_max)), $is_binary ? ZBX_KIBIBYTE : 1000))));
+	$power = $calc_power
+		? (int) min(8, max(0, floor(log(max(abs($scale_min), abs($scale_max)), $is_binary ? ZBX_KIBIBYTE : 1000))))
+		: 0;
 
 	$best_result_value = null;
 	$best_result = [
@@ -811,8 +824,8 @@ function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_
 	];
 
 	for ($rows = $rows_min; $rows <= $rows_max; $rows++) {
-		$clearance_min = $rows * 0.05;
-		$clearance_max = $rows * 0.1;
+		$clearance_min = min(0.5, $rows * 0.05);
+		$clearance_max = min(1, $rows * 0.1);
 
 		foreach (yieldGraphScaleInterval($scale_min, $scale_max, $is_binary, $power, $rows) as $interval) {
 			if ($interval == INF) {
@@ -841,6 +854,10 @@ function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_
 			$min = truncateFloat($min);
 			$max = truncateFloat($max);
 
+			if (is_infinite($min) || is_infinite($max)) {
+				break;
+			}
+
 			if ($min > $scale_min || $max < $scale_max) {
 				continue;
 			}
@@ -861,6 +878,7 @@ function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_
 				'power' => $power
 			];
 
+			// Expression optimized to avoid overflow.
 			$result_value = ($scale_min - $min) / $interval + ($max - $scale_max) / $interval;
 
 			if ($best_result_value === null || $result_value < $best_result_value) {
@@ -894,7 +912,10 @@ function calculateGraphScaleValues(float $min, float $max, bool $min_calculated,
 		string $units, bool $is_binary, int $power, int $precision_max): array {
 	$unit_base = $is_binary ? ZBX_KIBIBYTE : 1000;
 
-	$units_length = ($units === '' && $power == 0) ? 0 : (1 + mb_strlen($units) + ($power > 0 ? 1 : 0));
+	$units_length = ($units !== '' && $units !== '!')
+		? ($power > 0 ? 1 : 0) + mb_strlen($units) + ($units[0] !== '!' ? 1 : 0)
+		: ($power > 0 ? 1 : 0);
+
 	$precision = max(3, $units_length == 0 ? $precision_max : ($precision_max - $units_length - ($min < 0 ? 1 : 0)));
 
 	$decimals = min(ZBX_UNITS_ROUNDOFF_SUFFIXED, $precision - 1);
